package Intento1;

/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */



import java.util.logging.Level;
import java.util.logging.Logger;

import com.jme.app.BaseGame;
import com.jme.bounding.BoundingBox;
import com.jme.image.Texture;
import com.jme.input.KeyBindingManager;
import com.jme.input.KeyInput;
import com.jme.math.Vector3f;
import com.jme.renderer.Camera;
import com.jme.renderer.ColorRGBA;
import com.jme.scene.Node;
import com.jme.scene.shape.Sphere;
import com.jme.scene.state.TextureState;
import com.jme.system.DisplaySystem;
import com.jme.system.JmeException;
import com.jme.util.TextureManager;
import com.jme.util.Timer;

/**
 * <code>Tutorial 2</code> shows how to build our own Application
 * framework for Flag Rush.
 * For Flag Rush Tutorial Series.
 * @author Mark Powell
 */
public class Leccion2 extends BaseGame {
    private static final Logger logger = Logger.getLogger(Leccion2.class
            .getName());
        public static final String LOGO_AUTENTIA = "Intento1/imagenes/Autentia.jpg";

	protected Timer timer;
	//Our camera object for viewing the scene
	private Camera cam;
	//the root node of the scene graph
	private Node scene;
	//TextureState to show the monkey on the sphere.
	private TextureState ts;
	//display attributes for the window. We will keep these values
	//to allow the user to change them
	private int width, height, depth, freq;
	private boolean fullscreen;

	/**
	 * Main entry point of the application
	 */
	public static void main(String[] args) {
		Leccion2 app = new Leccion2();
		//We will load our own "fantastic" Flag Rush logo. Yes, I'm an artist.
		app.setConfigShowMode(ConfigShowMode.AlwaysShow, Leccion2.class.getClassLoader()
				.getResource(LOGO_AUTENTIA));
		app.start();
	}

	/**
	 * During an update we only look for the escape button and update the timer
	 * to get the framerate.
	 *
	 * @see com.jme.app.BaseGame#update(float)
	 */
	protected void update(float interpolation) {
		//update the time to get the framerate
		timer.update();
		interpolation = timer.getTimePerFrame();
		//if escape was pressed, we exit
		if (KeyBindingManager.getKeyBindingManager().isValidCommand("exit")) {
			finished = true;
		}
	}

	/**
	 * draws the scene graph
	 *
	 * @see com.jme.app.BaseGame#render(float)
	 */
	protected void render(float interpolation) {
		//Clear the screen
		display.getRenderer().clearBuffers();

		display.getRenderer().draw(scene);

	}

	/**
	 * initializes the display and camera.
	 *
	 * @see com.jme.app.BaseGame#initSystem()
	 */
	protected void initSystem() {
		//store the settings information
		width = settings.getWidth();
		height = settings.getHeight();
		depth = settings.getDepth();
		freq = settings.getFrequency();
		fullscreen = settings.isFullscreen();

		try {
			display = DisplaySystem.getDisplaySystem(settings.getRenderer());
			display.createWindow(width, height, depth, freq, fullscreen);

			cam = display.getRenderer().createCamera(width, height);
		} catch (JmeException e) {
            logger.log(Level.SEVERE, "Could not create displaySystem", e);
			System.exit(1);
		}

		//set the background to black
		display.getRenderer().setBackgroundColor(ColorRGBA.black.clone());

		//initialize the camera
		cam.setFrustumPerspective(45.0f, (float)width / (float)height, 1, 1000);
		Vector3f loc = new Vector3f(0.0f, 0.0f, 25.0f);
		Vector3f left = new Vector3f(-1.0f, 0.0f, 0.0f);
		Vector3f up = new Vector3f(0.0f, 1.0f, 0.0f);
		Vector3f dir = new Vector3f(0.0f, 0f, -1.0f);
		// Move our camera to a correct place and orientation.
		cam.setFrame(loc, left, up, dir);
		/** Signal that we've changed our camera's location/frustum. */
		cam.update();

		 /** Get a high resolution timer for FPS updates. */
	    timer = Timer.getTimer();

		display.getRenderer().setCamera(cam);

		KeyBindingManager.getKeyBindingManager().set("exit",
				KeyInput.KEY_ESCAPE);
	}

	/**
	 * initializes the scene
	 *
	 * @see com.jme.app.BaseGame#initGame()
	 */
	protected void initGame() {
		scene = new Node("Scene graph node");
		cam.update();

		//Create our Sphere
		Sphere s = new Sphere("Sphere", 30, 30, 25);
		s.setLocalTranslation(new Vector3f(0, 0, -40));
		s.setModelBound(new BoundingBox());
		s.updateModelBound();

		ts = display.getRenderer().createTextureState();
		ts.setEnabled(true);
		ts.setTexture(TextureManager.loadTexture(Leccion2.class.getClassLoader()
				.getResource(LOGO_AUTENTIA),
				Texture.MinificationFilter.Trilinear, Texture.MagnificationFilter.Bilinear));

		s.setRenderState(ts);

		scene.attachChild(s);

		//update the scene graph for rendering
		scene.updateGeometricState(0.0f, true);
		scene.updateRenderState();
	}

	/**
	 * will be called if the resolution changes
	 *
	 * @see com.jme.app.BaseGame#reinit()
	 */
	protected void reinit() {
		display.recreateWindow(width, height, depth, freq, fullscreen);
	}

    protected void quit() {
        super.quit();
        System.exit(0);
    }

	/**
	 * clean up the textures.
	 *
	 * @see com.jme.app.BaseGame#cleanup()
	 */
	protected void cleanup() {
		ts.deleteAll();

	}
}
