package Intento1;

import com.jme.bounding.BoundingBox;
import com.jme.image.Texture;
import com.jme.math.FastMath;
import com.jme.math.Quaternion;
import com.jme.math.Vector3f;
import com.jme.renderer.Renderer;
import com.jme.scene.Node;
import com.jme.scene.SharedMesh;
import com.jme.scene.shape.Box;
import com.jme.scene.shape.Cylinder;
import com.jme.scene.state.BlendState;
import com.jme.scene.state.TextureState;
import com.jme.system.DisplaySystem;
import com.jme.util.TextureManager;



public class escenario extends Node {
	private static final long serialVersionUID = 1L;

   
    private Texture t;

    
    public escenario(String name) {
        super(name);
        buildFence();
    }

    /**
     * Tiene este metodo ya que se creara una especie de campo y con este metodo
     * lo podemos animar
     */
    public void update(float interpolation) {
        /**
         * Se usara el valor de interpolation para mantener la velocidad del
         * campo de fuerza constante.
         */
         t.getTranslation().y += 0.3f * interpolation;
        
        if(t.getTranslation().y > 1) {
            t.getTranslation().y = 0;
        }
    }

    /**
     * buildFence crea la geometria de la valla
     */
    private void buildFence() {
        
        Cylinder postGeometry = new Cylinder("post", 10, 10, 1, 10);
        Quaternion q = new Quaternion();
        q.fromAngleAxis(FastMath.PI/2, new Vector3f(1,0,0));
        postGeometry.setLocalRotation(q);
        postGeometry.setModelBound(new BoundingBox());
        postGeometry.updateModelBound();
        /**
         * creamos los 4 postes, no es buena idea que compartamos la estructura
         * asi que usaremos los valores locales
         */
        SharedMesh post1 = new SharedMesh("post1", postGeometry);
        post1.setLocalTranslation(new Vector3f(0,0.5f,0));
        SharedMesh post2 = new SharedMesh("post2", postGeometry);
        post2.setLocalTranslation(new Vector3f(32,0.5f,0));
        SharedMesh post3 = new SharedMesh("post3", postGeometry);
        post3.setLocalTranslation(new Vector3f(0,0.5f,32));
        SharedMesh post4 = new SharedMesh("post4", postGeometry);
        post4.setLocalTranslation(new Vector3f(32,0.5f,32));

        /**
         * este sera el cilindro maestro qe mantendra el campo de fuerza
         * en su sitio
         */

        Cylinder strutGeometry = new Cylinder("strut", 10,10, 0.125f, 32);
        strutGeometry.setModelBound(new BoundingBox());
        strutGeometry.updateModelBound();

        /**
         * Otra vez tenemos que compartir esta estrucutra asi qeu la rotaremos
         * para hacer las conextsiones
         */
        SharedMesh strut1 = new SharedMesh("strut1", strutGeometry);
        Quaternion rotate90 = new Quaternion();
        rotate90.fromAngleAxis(FastMath.PI/2, new Vector3f(0,1,0));
        strut1.setLocalRotation(rotate90);
        strut1.setLocalTranslation(new Vector3f(16,3f,0));
        SharedMesh strut2 = new SharedMesh("strut2", strutGeometry);
        strut2.setLocalTranslation(new Vector3f(0,3f,16));
        SharedMesh strut3 = new SharedMesh("strut3", strutGeometry);
        strut3.setLocalTranslation(new Vector3f(32,3f,16));
        SharedMesh strut4 = new SharedMesh("strut4", strutGeometry);
        strut4.setLocalRotation(rotate90);
        strut4.setLocalTranslation(new Vector3f(16,3f,32));

        /**
         * Crea el campo de fuerza
         * La primera caja controla el eje x, la segunda el eje z.
         * No se rota la caja para enseñar que se pueden crear de varias formas.
         */
        Box forceFieldX = new Box("forceFieldX", new Vector3f(-16, -3f, -0.1f), new Vector3f(16f, 3f, 0.1f));
        forceFieldX.setModelBound(new BoundingBox());
        forceFieldX.updateModelBound();
        /**
         * Tambien compartiremos estas cajas
         */
        SharedMesh forceFieldX1 = new SharedMesh("forceFieldX1",forceFieldX);
        forceFieldX1.setLocalTranslation(new Vector3f(16,0,0));
        SharedMesh forceFieldX2 = new SharedMesh("forceFieldX2",forceFieldX);
        forceFieldX2.setLocalTranslation(new Vector3f(16,0,32));

        
        Box forceFieldZ = new Box("forceFieldZ", new Vector3f(-0.1f, -3f, -16), new Vector3f(0.1f, 3f, 16));
        forceFieldZ.setModelBound(new BoundingBox());
        forceFieldZ.updateModelBound();
        SharedMesh forceFieldZ1 = new SharedMesh("forceFieldZ1",forceFieldZ);
        forceFieldZ1.setLocalTranslation(new Vector3f(0,0,16));
        SharedMesh forceFieldZ2 = new SharedMesh("forceFieldZ2",forceFieldZ);
        forceFieldZ2.setLocalTranslation(new Vector3f(32,0,16));

        /**
         * se añade todos los campos de fuerza a un solo nodo y se le hace parte
         * de la zona transparente.
         */

        Node forceFieldNode = new Node("forceFieldNode");
        forceFieldNode.setRenderQueueMode(Renderer.QUEUE_TRANSPARENT);
        forceFieldNode.attachChild(forceFieldX1);
        forceFieldNode.attachChild(forceFieldX2);
        forceFieldNode.attachChild(forceFieldZ1);
        forceFieldNode.attachChild(forceFieldZ2);

        /**
         * se añade los valores alfa para el nodo transparente
         */

        BlendState as1 = DisplaySystem.getDisplaySystem().getRenderer().createBlendState();
        as1.setBlendEnabled(true);
        as1.setSourceFunction(BlendState.SourceFunction.SourceAlpha);
        as1.setDestinationFunction(BlendState.DestinationFunction.One);
        as1.setTestEnabled(true);
        as1.setTestFunction(BlendState.TestFunction.GreaterThan);
        as1.setEnabled(true);

        forceFieldNode.setRenderState(as1);

        /**
         * cargamos la textura para el campo de fuerza
         */

        TextureState ts = DisplaySystem.getDisplaySystem().getRenderer().createTextureState();
        t = TextureManager.loadTexture(Leccion2.class.getClassLoader()
                  .getResource("Intento1/texturas/reflector.jpg"),
                  Texture.MinificationFilter.Trilinear, Texture.MagnificationFilter.Bilinear);

        t.setWrap(Texture.WrapMode.Repeat);
        t.setTranslation(new Vector3f());
        ts.setTexture(t);

        forceFieldNode.setRenderState(ts);

        /**
         * ponemos todas las postes en el nodo de una torre
         */

        Node towerNode = new Node("tower");
        towerNode.attachChild(post1);
        towerNode.attachChild(post2);
        towerNode.attachChild(post3);
        towerNode.attachChild(post4);

        /**
         * se añade la torre a la zona opaca ya que no queremos que se vea a
         * traves de ella
         */

        towerNode.setRenderQueueMode(Renderer.QUEUE_OPAQUE);

        /**
         * cargamos la textura para las torres
         */

        TextureState ts2 = DisplaySystem.getDisplaySystem().getRenderer().createTextureState();
        Texture t2 = TextureManager.loadTexture(Leccion2.class.getClassLoader()
                  .getResource("Intento1/texturas/post.jpg"),
                  Texture.MinificationFilter.Trilinear, Texture.MagnificationFilter.Bilinear);

        ts2.setTexture(t2);

        towerNode.setRenderState(ts2);

        /**
         * ponemos todas las estructuras en un solo nodo
         */

        Node strutNode = new Node("strutNode");
        strutNode.attachChild(strut1);
        strutNode.attachChild(strut2);
        strutNode.attachChild(strut3);
        strutNode.attachChild(strut4);
        /**
         * Esto tambien se encuentra en la zona opaca
         */
        strutNode.setRenderQueueMode(Renderer.QUEUE_OPAQUE);

        /**
         * cargamos la textura de las estructuras
         */
        TextureState ts3 = DisplaySystem.getDisplaySystem().getRenderer().createTextureState();
        Texture t3 = TextureManager.loadTexture(Leccion2.class.getClassLoader()
                  .getResource("Intento1/texturas/rust.jpg"),
                  Texture.MinificationFilter.Trilinear, Texture.MagnificationFilter.Bilinear);

        ts3.setTexture(t3);

        strutNode.setRenderState(ts3);

        /**
         * añadimos todas las piezas en el nodo principal de la valla
         */

        this.attachChild(forceFieldNode);
        this.attachChild(towerNode);
        this.attachChild(strutNode);
    }
}
